"use strict";

var _require = require('./packageData'),
    oldPresets = _require.presets,
    oldPlugins = _require.plugins;

var upgradeOptions = require('./upgradeOptions');

function changeName(originalName, kind) {
  var oldNames = kind === 'plugin' ? oldPlugins : oldPresets;
  var name = originalName;

  if (name.indexOf(`babel-${kind}`) !== 0 && name.indexOf('@babel/') !== 0) {
    name = `babel-${kind}-${name}`;
  }

  if (oldNames.hasOwnProperty(name)) {
    if (oldNames[name]) {
      return oldNames[name];
    } else {
      return null;
    }
  }

  return originalName;
} // TODO: fix all of this


function changePresets(config) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var presets = config.presets;

  if (!Array.isArray(presets) && typeof presets === 'string') {
    presets = config.presets = config.presets.split(',').map(function (preset) {
      return preset.trim();
    });
  } // check if presets are there


  if (presets) {
    // assume it's an array
    for (var i = 0; i < presets.length; i++) {
      var preset = presets[i];
      var presetsToReplace = Object.keys(oldPresets); // check if it's a preset with options (an array)

      var isArray = Array.isArray(preset);
      var name = changeName(isArray ? preset[0] : preset, 'preset');

      if (name === null) {
        presets.splice(i, 1);
        i--;
      } else {
        if (isArray) preset[0] = name;else preset = name;
        presets[i] = upgradeOptions(preset);
      }
    }

    if (options.hasFlow && !presets.includes('@babel/preset-flow')) {
      presets.push('@babel/preset-flow');
    }
  }
}

function changePlugins(config) {
  var plugins = config.plugins;

  if (!Array.isArray(plugins) && typeof plugins === 'string') {
    plugins = config.plugins = config.plugins.split(',').map(function (plugin) {
      return plugin.trim();
    });
  } // check if plugins are there


  if (plugins) {
    // assume it's an array
    for (var i = 0; i < plugins.length; i++) {
      var plugin = plugins[i]; // check if it's a plugin with options (an array)

      var isArray = Array.isArray(plugin);
      var name = changeName(isArray ? plugin[0] : plugin, 'plugin');

      if (name === null) {
        plugins.splice(i, 1);
        i--;
      } else {
        if (isArray) plugin[0] = name;else plugin = name;
        plugins[i] = upgradeOptions(plugin);
      }
    }
  }
}

module.exports = function upgradeConfig(config, options) {
  config = Object.assign({}, config);
  changePresets(config, options);
  changePlugins(config);

  if (config.env) {
    Object.keys(config.env).forEach(function (env) {
      changePresets(config.env[env]);
      changePlugins(config.env[env]);
    });
  }

  return config;
};